#include <Arduino.h>
#include <math.h>
#include "whetstone.h"


void checkcmd(void);
void printtempvbat(void);
//void enablefpu(void);
void cosfade();
void ARTtoggle();

int ledPin = LED_BUILTIN;
int print = false;

//used by cosfade()
#define PER 20
#define REP 10
int8_t led = 0;
int n = PER, cnt = 0;
uint8_t p = 0;

// the setup() method runs once when the sketch starts
void setup() {

// not needed fpu is enabled in the core
//	enablefpu();

	//initialize the digital pin as an output:
	pinMode(ledPin, OUTPUT);
	digitalWrite(ledPin, LOW);

	//these are codes for steve's libmaple core, left here as a ref
	// ADC_CCR_VBATE;  // f401 allows only vbat or temp sensor read, setting VBATE reads vbat
	//ADC_COMMON->CCR |= ADC_CCR_TSVREFE; //| ADC_CCR_VBATE;    // enable VREFINT, VBAT or temp sensor
	//adc_calibrate(ADC1);
	//adc_set_sampling_time(ADC1, ADC_SMPR_480); // sample time for temp sensor

	pinMode(ATEMP, INPUT_ANALOG);
	pinMode(AVREF, INPUT_ANALOG);

}


// Enable the FPU (Cortex-M4 - STM32F4xx and higher)
// http://infocenter.arm.com/help/topic/com.arm.doc.dui0553a/BEHBJHIG.html
//void enablefpu() {
//	  __asm volatile
//	  (
//	    "  ldr.w r0, =0xE000ED88    \n"  /* The FPU enable bits are in the CPACR. */
//	    "  ldr r1, [r0]             \n"  /* read CAPCR */
//	    "  orr r1, r1, #( 0xf << 20 )\n" /* Set bits 20-23 to enable CP10 and CP11 coprocessors */
//	    "  str r1, [r0]              \n" /* Write back the modified value to the CPACR */
//	    "  dsb                       \n" /* wait for store to complete */
//	    "  isb"                          /* reset pipeline now the FPU is enabled */
//	  );
//}

//the loop() method runs over and over again, 
//as long as maple has power
void loop() {

	checkcmd();
	if (print && n == 0 && cnt == 0) printtempvbat();
	cosfade();
}


void cosfade() {
  digitalWrite(LED_BUILTIN,led);
  led = ~led & 1;
  if(led)
    delay(PER-p);
  else
    delay(p);
  if(cnt>REP) {
	  float nf, perf;
	  n = n>PER?0:n+1;
	  nf = n * 1.0;
	  perf = PER * 1.0;
	  p = perf * ( cos(2 * PI * nf/perf) / 2.0 + 0.5 );
	  cnt=0;
  } else
	  cnt++;

}


void printtempvbat() {
	//ADC1 channel 18 is vrefint 1.23v
	//uint16_t vrefint = adc_read(ADC1, 17);
	uint16_t vrefint = analogRead(AVREF);
	Serial.print("Vref int (1.21v):");
	Serial.print(vrefint);
	Serial.println();

	//ADC1 channel 16 is temperature sensor
	//uint16_t vtemp = adc_read(ADC1, 18);
	uint16_t vtemp = analogRead(ATEMP);
	Serial.print("temp sensor:");
	Serial.print(vtemp);
	Serial.println();

	uint16_t mv = (1210 * vtemp) / vrefint;
	Serial.print("mvolt:");
	Serial.print(mv);
	Serial.println();

	// specs 5.3.22 temp sensor characteristics
	// V 25 deg ~ 0.76v
	// slope 2.5 mv/C
	uint16_t v25 = 760;
	float temp = (mv - v25) * 1.0 / 2.5 + 25.0;
	Serial.print("temp:");
	Serial.print(temp);
	Serial.println();

	/* for stm32f401 it is either temp or vbat
	//ADC1 channel 17 is VBAT / 2
	uint16_t vbatsens = adc_read(ADC1, 18);
	Serial.print("VBAT/2 sensor:");
	Serial.print(vbatsens);
	Serial.println();

	uint16_t vbat = vbatsens * 2.0 * 1230 / vrefint;
	Serial.print("VBAT (mv):");
	Serial.print(vbat);
	Serial.println();
	*/

}


void checkcmd() {
	if(Serial.available()) {
		char r = Serial.read();
		if(r=='p') {
			print = true;
		} else if (r=='s') {
			print = false;
		} else if (r=='a') {
			ARTtoggle();
		} else if (r=='w') {
			whetstone(F_CPU/1000000);
		}
	}
}

void blinks() {
	for (int i = 0; i < 5; i++) {
		digitalWrite(LED_BUILTIN, HIGH);
		delay(100);
		digitalWrite(LED_BUILTIN, LOW);
		delay(100);
	}
}

void ARTtoggle() {

	if((FLASH->ACR & FLASH_ACR_ICEN)!=FLASH_ACR_ICEN) { // art disabled
		/* enable the ART accelerator */
		/* enable prefetch buffer */
		FLASH->ACR |= FLASH_ACR_PRFTEN;
		/* Enable flash instruction cache */
		FLASH->ACR |= FLASH_ACR_ICEN;
		/* Enable flash data cache */
		FLASH->ACR |= FLASH_ACR_DCEN;
		asm("wfi");
		Serial.println("ART enabled");
	} else {
		/* disable the ART accelerator */
		/* disable flash instruction cache */
		FLASH->ACR &= ~FLASH_ACR_ICEN;
		/* disable flash data cache */
		FLASH->ACR &= ~FLASH_ACR_DCEN;
		/* enable prefetch buffer */
		FLASH->ACR |= FLASH_ACR_PRFTEN;
		asm("wfi");
		Serial.println("ART disabled");
	}
}


